"""Plot a matplotlib fig on a tkinter widget win (tkinter.Tk() or tkinter.Frame)."""
from typing import Optional, Union
import tkinter as tk
import tkinter.ttk as ttk
import matplotlib
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk


def tk_plot(
    fig: matplotlib.figure.Figure,
    win: Union[tk.Tk, ttk.Frame] = None,
    title: str = "",
    toolbar: bool = True,
) -> FigureCanvasTkAgg:
    r"""Plot matplotlib figues in a tkinter.Tk/Frame.

    Args:
        fig: matplotlib figure
        win: tkinter.Tk() or tkinter.Frame()
        title: tkinter widget title
        toolbar: if set to True, a toolbar (matplotlib toolbar) is shown

    Returns:
        canvas: a FigureCanvasTkAgg, run canvas.draw() to update.

    snippets\tkinter drawfig*.py (PySimpleGUi matplotlib exampl)
    matplotlib cookbook: Embedding Matplotlib in a Tkinter GUI application

    matplotlib.use('TkAgg')

    import seaborn as sns
    sns.set_theme()
    sns.set_style("whitegrid")
    matplotlib.pyplot.ion()  # for testing in ipython

    from matplotlib.gridspec import GridSpec

    # gs = GridSpec(3, 1)
    # gs = fig.add_gridspec(2, 2)
    # ax = fig.add_subplot(gs[0, :])

    import numpy as np
    ax = fig.add_subplot()
    t = np.arange(0, 3, .01)
    ax.plot(t, 2 * np.sin(2 * np.pi * t))
    canvas.draw()

    # fig.clf()
    ax.cla()
    ax = fig.add_subplot(gs[0, :])
    ax.plot(np.sin(2 * np.pi * t))
    canvas.draw()
    """
    if win is None:
        win = tk.Tk()
        if title:
            win.wm_title(title)
        else:
            win.wm_title("Fig in Tk")

    canvas = FigureCanvasTkAgg(fig, win)
    canvas.get_tk_widget().pack(side="top", fill="both", expand=1)
    if toolbar:
        NavigationToolbar2Tk(canvas, win)

    tk_plot.win = win
    # tk_plot.win.destroy() to exit

    return canvas


def main(root):
    """Run."""
    win = ttk.Frame(root)
    win.pack()

    fig = matplotlib.figure.Figure(figsize=(5, 4), dpi=100)

    # canvas = tk_plot(fig)
    # tk_plot.win.destroy()  # to kill

    canvas = tk_plot(fig, win)

    canvas.draw()
    canvas.cla()  # to clear the canvas

    _ = """
    # update fig
    ax1 = fig.add_subplot(211)  # gs[,] gs = mpl.gridspec.GridSpec(3,1)
    # ax1.plot(...) or sns.heatmap(ax=ax1), df.plot(..., ax=ax)
    canvas.draw()

    # fig.clear()  # fig.clf()
    #
    # ax1.cla()
    """


if __name__ == "__main__":
    # in ipython
    # %matplotlib tk
    root = tk.Tk()
    root.wm_title("Fig in Tk")

    main(root)
    root.mainloop()



"""Calculate corr matrix for given en and zh.

en = loatext(filename)

en_p = process_en(en)
en_tr = en2zh(en_p)

s = fast_scores(en_tr, zh)

plt.ion()
sns.set()
sns.set_style("darkgrid")
sns.heatmap(s, cmap="viridis_r").invert_yaxis()

aset = cmat2aset(s)

df = pd.DataFrame(aset, columns=list("xyz")).replace("", np.nan).dropna()
df.plot.scatter("x", "y", c="z")

"""



"""Insert spaces, mypython/split_chinese.py."""
import re


def insert_spaces(text: str) -> str:
    """Insert space in Chinese characters.

    >>> insert_spaces("test亨利it四世上")
    ' test 亨 利 it 四 世 上 '
    >>> insert_spaces("test亨利it四世上").strip().__len__()
    17

    """
    return re.sub(r"(?<=[a-zA-Z\d]) (?=[a-zA-Z\d])", "", text.replace("", " "))



"""Process w4wtr english to chinese."""
from typing import List, Union

# from pathlib import Path
# import msgpack
import nltk
from nltk.corpus import stopwords
from simplemma import lemmatize, load_data

# nltk.download("stopwords")
# do this
# python -m nltk.downloader stopwords
# or

try:
    stopwords_ = set(stopwords.words("english"))
except Exception: # download when use the first time
    nltk.download("stopwords")
    stopwords_ = set(stopwords.words("english"))

langdata = load_data("en")


# fmt: off
def process_en(
        text: Union[str, List[str]],
        remove_en_stopwords: bool = True,
        lemma: bool = True,
        # remove_zh_stopwords: bool = True,
        dedup: bool = True,
) -> List[List[str]]:
    #  fmt: on
    """Process w4wtr english to chinese.

    Args:
        text: english text to process_en
        remove_en_stopwords: remove stopwords
        lemma: lemmatize english ussing simplemma.text_lemmatize
        dedup: remove duplicate chinese chars

    Returns:
        w4w split text

    """
    if isinstance(text, str):
        res = [text]
    else:
        res = text.copy()

    def remove_(elm):
        if isinstance(elm, str):
            elm = elm.split()
        return [word for word in elm if word.lower() not in stopwords_]

    # remove_en_stopwords
    res = [*map(str.split, res)]
    if remove_en_stopwords:
        res = [*map(remove_, res)]

    # simplemma.lemmatize or str.lower
    if lemma:
        res = [[*map(lambda x: lemmatize(x, langdata), line)] for line in res]
    else:
        res = [[*map(str.lower, line)] for line in res]

    # dedup
    if dedup:
        res = [list(set(line)) for line in res]

    return res



"""Normalize chinese text and insert spaces beteen chars."""
from typing import List, Union

import re

# from logzero import logger

# from fast_scores.insert_spaces import insert_spaces
# from fast_scores.split_chinese import split_chinese


# fmt: off
def process_zh(
        text: Union[str, List[str]],
        remove_nonchinese: bool = False,
        dedup: bool = False,
        # remove_stopwords: bool = False,  ?na
) -> List[str]:
    # fmt: on
    """Normalize chinese text and insert spaces beteen chars.

    Args
        text: chinese to process
        remove_nonchinese: remove or not
        dedup: deduplicate or not

    Returns
        processed text
    """
    if isinstance(text, str):
        text = [text]

    res = text.copy()

    if remove_nonchinese:
        patt = re.compile(r"[^一-龟]")
        res = map(lambda x: re.sub(patt, "", x), res)

    # insert spaces and split
    # res = [insert_spaces(elm) for elm in res]
    # res = [split_chinese(elm) for elm in res]

    if dedup:
        # res = [list(set(elm)) for elm in res]
        res = ["".join(set(elm)) for elm in res]
    else:
        res = [*res]

    return res



"""Translate a word via mdx_dict."""
# from typing import List, Union

from pathlib import Path
import msgpack
from string import punctuation

_ = Path(__file__).parent
_ = Path(_, "msbing_c_e.msgpk")
mdx_dict = msgpack.load(open(_, "rb"))


def word_tr(word: str, strip: bool = True) -> str:
    """Translate a word via mdx_dict.

    Args:
        word: text (str) to translate
        strip: when True, word.strip(string.punctuation)

    Returns:
        "".join(mdx_dict.get(
            word.strip(punctuation + "\r\n\t ").lower(), {}
        ).values())
    """
    try:
        word = str(word).lower()
    except Exception as exc:
        print("Something is terribly wrong: %s" % exc)
        raise exc

    if strip:
        word = word.strip(punctuation + "\r\n\t ")
        return "".join(mdx_dict.get(word, {'': word}).values())

    return "".join(mdx_dict.get(word, {'': word}).values())



"""Calculate fast_scores."""
from typing import List

import numpy as np
from sklearn.feature_extraction.text import TfidfVectorizer
from logzero import logger

from fast_scores.cos_matrix2 import cos_matrix2
from fast_scores.gen_model import gen_model


# fmt: off
def fast_scores(
        text1: List[str],
        text2: List[str],
        model: TfidfVectorizer = None,
        dot: bool = True,
) -> np.ndarray:
    # fmt: on
    """Calculate fast_scores.

    Args:
        text1: chinese text
        text2: chinese text

    Returns:
        correlation matrix (float)
    """
    # logger.debug(" entry ")

    # create model on the fly
    if model is None:
        logger.info("No model provided, creating one on the fly")
        try:
            # model = TfidfVectorizer().fit(text1 + text2)
            model = gen_model(text1 + text2)
        except Exception as e:
            logger.error(e)
            raise SystemExit(1) from e
        logger.info("Done creating model")
        # shake 10000x10000 ~7s-16s

    vec1 = model.transform(text1)  # scipy.sparse.csr.csr_matrix
    vec2 = model.transform(text2)

    # supposed to be much faster
    if dot:
        return vec1.dot(vec2.T).toarray().T
    # shake 10000x11000 41.1 s ± 15.5 s
    # shake 5000x5100 11s
    # shake 2000x2100 3.7s
    # shake 1000x1100 2.1s
    # shake 10000x10100 3min 1min 13s
    # comfort zone length: j = math.sqrt(psutil.virtual_memory().available/64)
    # estimated time: j / 1000 * 2 s

    return cos_matrix2(vec1.toarray(), vec2.toarray()).T
    # shake 1000x1100 11s 21s
    # shake 2000x2100 45s
    # shake 10000x10000



"""Compute the cosine similarity matrix of A.

https://stackoverflow.com/questions/17627219/whats-the-fastest-way-in-python-to-calculate-cosine-similarity-given-sparse-mat
"""
import numpy as np


def cos_matrix2(mat1: np.ndarray, mat2: np.ndarray = None) -> np.ndarray:
    """Compute the cosine similarity matrix of mat1, mat2: mat1 * mat2.T.

    Args:
        mat1: np.asarray
        mat2: [Optional], if not present mat2 = mat1
    Returns
        cosine similarity
    """
    if not isinstance(mat1, np.ndarray):
        mat1 = np.asarray(mat1, dtype=np.float32)

    if mat2 is None:
        mat2 = mat1.copy()

    if not isinstance(mat2, np.ndarray):
        mat2 = np.asarray(mat2, dtype=np.float32)

    if mat1.shape[1] != mat2.shape[1]:
        print("shape mismatch: %s, %s", mat1.shape, mat2.shape)
        raise SystemError(1)
    cosine = np.dot(mat1, mat2.T)

    norm1 = np.linalg.norm(mat1, axis=1)
    norm2 = np.linalg.norm(mat2, axis=1)

    # if not (norm1 and norm2): return 0

    size1 = norm1.size
    size2 = norm2.size
    norm_mat = np.dot(norm1.reshape(size1, 1), norm2.reshape(1, size2))

    for idx in range(size1):
        for jdx in range(size2):
            if norm_mat[idx, jdx] == 0:
                cosine[idx, jdx] = 0.0
            else:
                cosine[idx, jdx] = cosine[idx, jdx] / norm_mat[idx, jdx]
    return cosine



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fast_scores']

package_data = \
{'': ['*']}

install_requires = \
['logzero>=1.7.0,<2.0.0',
 'msgpack>=1.0.2,<2.0.0',
 'nltk>=3.6.2,<4.0.0',
 'numpy>=1.21.0,<2.0.0',
 'simplemma>=0.3.0,<0.4.0',
 'sklearn>=0.0,<0.1']

setup_kwargs = {
    'name': 'fast-scores',
    'version': '0.1.3a0',
    'description': ' ',
    'long_description': '# fast-scores\n[![tests](https://github.com/ffreemt/fast-scores/actions/workflows/routine-tests.yml/badge.svg)][![python](https://img.shields.io/static/v1?label=python+&message=3.7%2B&color=blue)](https://img.shields.io/static/v1?label=python+&message=3.7%2B&color=blue)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/fast_scores.svg)](https://badge.fury.io/py/fast_scores)\n\nCalculate correlatioin matrix fast\n\n## Usage\n\n```shell\n\n```',
    'author': 'freemt',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ffreemt/fast-scores',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
